unit Orders;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.Grids, AdvObj, BaseGrid, AdvGrid, MSAccess,
  syncObjs, uOrderClass, generics.Collections;

type
  TfrmOrders = class(TForm)
    gridorders: TAdvStringGrid;
    gridOrderLines: TAdvStringGrid;
    procedure gridordersGetDisplText(Sender: TObject; ACol, ARow: Integer;
      var Value: string);
    procedure gridordersRowChanging(Sender: TObject; OldRow, NewRow: Integer;
      var Allow: Boolean);
    procedure gridOrderLinesGetDisplText(Sender: TObject; ACol, ARow: Integer;
      var Value: string);
  private
    { Private declarations }
    fConnection : TMSConnection;
    OrderThread : TOrderThread;
    orders : TObjectList<TOrder>;
    procedure getData;
    procedure DataGotten(Sender: TObject);
  public
    { Public declarations }
    class Procedure OpenOrderform(aPopupParent : TForm; aConnection : TMSConnection);
  end;

var
  orderCrit : TCriticalSection;

implementation

{$R *.dfm}

{ TfrmOrders }

procedure TfrmOrders.getData;
begin
  OrderThread := TOrderThread.Create(true);
  OrderThread.gridOrders := self.gridorders;
  OrderThread.OrderList := self.orders;
  OrderThread.OnTerminate := DataGotten;
  OrderThread.Start;
end;

procedure TfrmOrders.gridOrderLinesGetDisplText(Sender: TObject; ACol,
  ARow: Integer; var Value: string);
var
  aOrderLine : TOrderLine;
begin
  if aRow > 0 then
  begin
    if gridOrderLines.Objects[0, arow] is TOrderLine then
    begin
      aOrderLine := TOrderLine(gridOrderLines.Objects[0, aRow]);
      case aCol of
        0 : Value := IntToStr(aOrderLine.OrderNo);
        1 : Value := IntToStr(aOrderLine.ItemNo);
        2 : Value := IntToStr(aOrderLine.Quantity);
        3 : Value := aOrderLine.DiscountAsString;
      end;
    end;
  end;
end;

procedure TfrmOrders.gridordersGetDisplText(Sender: TObject; ACol,
  ARow: Integer; var Value: string);
var
  aOrder : TOrder;
begin
  if aRow > 0 then
  begin
    if gridOrders.Objects[0, aRow] is TOrder then
    begin
      aOrder := TOrder(gridOrders.Objects[0, aRow]);
      case aCol of
        0 : Value := IntToStr(aOrder.OrderNo);
        1 : Value := aOrder.SaleDateAsString;
        2 : Value := IntToStr(aOrder.CustNo);
        3 : Value := aOrder.ShipToContact;

      end;
    end;
  end;
end;

procedure TfrmOrders.gridordersRowChanging(Sender: TObject; OldRow,
  NewRow: Integer; var Allow: Boolean);
var
  aOrder : TOrder;
  I: Integer;
begin
  if NewRow > 0 then
  begin
    if gridOrders.Objects[0, newRow] is TOrder then
    begin
      aOrder  := Torder(gridOrders.Objects[0, newRow]);
      gridOrderLines.ClearRows(1, gridOrderLines.RowCount);
      gridOrderLines.RowCount := aOrder.OrderLines.Count + 1;
      for I := 0 to aOrder.OrderLines.Count - 1 do
      begin
        gridOrderLines.Objects[0, i + 1] := aOrder.OrderLines[i];
      end;
      gridOrderLines.Refresh;
    end;
  end;
end;

procedure TFrmOrders.DataGotten(Sender : TObject);
var
  dummy : Boolean;
begin
  gridOrders.SelectRows(1,1);
  gridOrders.Refresh;
  dummy := true;
  gridordersRowChanging(sender, 0, 1, dummy);
end;


class procedure TfrmOrders.OpenOrderform(aPopupParent : TForm; aConnection: TMSConnection);
var
  frmOrders: TfrmOrders;

begin
  frmOrders := TfrmOrders.Create(nil);
  try
    frmOrders.PopupParent := aPopupParent;
    frmOrders.orders := TObjectList<TOrder>.Create(true);
    frmOrders.getData;
    frmOrders.ShowModal;
  finally
    frmOrders.orders.Free;
    frmOrders.Release;
  end;
end;

initialization
  orderCrit := TCriticalSection.Create;


Finalization
  ordercrit.Destroy;

end.
