unit uCDSThread;

interface

uses
  Classes, DB, DBClient, AdvGrid, generics.collections, uEmployeeClass;

type
  TEmployeeThread = class(TThread)
    private
    FEmployeeList: TObjectList<TEmployee>;
    Fcds: TClientDataset;
    FcdsFileName: string;
    Fgrid: TAdvStringGrid;
    procedure Setcds(const Value: TClientDataset);
    procedure SetcdsFileName(const Value: string);
    procedure SetEmployeeList(const Value: TObjectList<TEmployee>);
    procedure Setgrid(const Value: TAdvStringGrid);
    protected
      procedure Execute; override;
    public
      property cds : TClientDataset read Fcds write Setcds;
      property cdsFileName : string read FcdsFileName write SetcdsFileName;
      property grid : TAdvStringGrid read Fgrid write Setgrid;
      property EmployeeList : TObjectList<TEmployee> read FEmployeeList write SetEmployeeList;
  end;


type
  TCdsThread = class(TThread)
  private
    Fcds: TClientDataSet;
    FcdsFilename: string;
    Fgrid: TAdvStringGrid;
    procedure Setcds(const Value: TClientDataSet);
    procedure SetcdsFilename(const Value: string);
    procedure Setgrid(const Value: TAdvStringGrid);
    { Private declarations }
  protected
    procedure Execute; override;
  Public
    property cds : TClientDataSet read Fcds write Setcds;
    property cdsFilename : string read FcdsFilename write SetcdsFilename;
    property grid : TAdvStringGrid read Fgrid write Setgrid;
  end;



implementation

{
  Important: Methods and properties of objects in visual components can only be
  used in a method called using Synchronize, for example,

      Synchronize(UpdateCaption);

  and UpdateCaption could look like,

    procedure TCdsThread.UpdateCaption;
    begin
      Form1.Caption := 'Updated in a thread';
    end;

    or

    Synchronize(
      procedure
      begin
        Form1.Caption := 'Updated in thread via an anonymous method'
      end
      )
    );

  where an anonymous method is passed.

  Similarly, the developer can call the Queue method with similar parameters as
  above, instead passing another TThread class as the first parameter, putting
  the calling thread in a queue with the other thread.

}

{ TCdsThread }
uses
  uDataView;

procedure TCdsThread.Execute;
var
  i, idx : integer;
begin
  { Place thread code here }

 // NameThreadForDebugging('boxthread');



  self.cds.loadFromFile( Self.cdsFilename);
  self.cds.Open;
  CritSec.Enter;
  self.grid.RowCount := self.cds.RecordCount +1;
  self.grid.ColCount := self.cds.FieldCount;
  CritSec.Leave;
  idx := 1;
  while not self.cds.eof do
  begin

    for I := 0 to self.grid.ColCount -1 do
    begin
      CritSec.Enter;
      self.grid.Cells[i, idx] := self.cds.Fields[i].AsString;
      CritSec.Leave;
    end;
    self.cds.Next;
    inc(idx);
    sleep(17);
  end;

end;

procedure TCdsThread.Setcds(const Value: TClientDataSet);
begin
  Fcds := Value;
end;

procedure TCdsThread.SetcdsFilename(const Value: string);
begin
  FcdsFilename := Value;
end;

procedure TCdsThread.Setgrid(const Value: TAdvStringGrid);
begin
  Fgrid := Value;
end;



{ TEmployeeThread }

procedure TEmployeeThread.Execute;
var
  i, idx : integer;
  aEmployee : TEmployee;
begin
  inherited;
  self.cds.loadFromFile( Self.cdsFilename);
  self.cds.Open;
  CritSec.Enter;
  self.grid.RowCount := self.cds.RecordCount +1;
  self.grid.ColCount := self.cds.FieldCount;
  CritSec.Leave;
  idx := 1;
  while not self.cds.eof do
  begin
    aEmployee := TEmployee.Create;
//    aEmployee.EmployeeNumber  := cds.FieldByName('EmpNo').AsInteger;
//    aEmployee.Lastname        := cds.FieldByName('LastName').AsString;
//    aEmployee.FirstName       := cds.FieldByName('FirstName' ).AsString;
//    aEmployee.PhoneExtenstion := cds.FieldByName('PhoneExt').AsString;
//    aEmployee.HireDate        := cds.FieldByName('HireDate').AsDateTime;
    aEmployee.LoadFromDataset(cds);
    self.FEmployeeList.Add(aEmployee);
    CritSec.Enter;
    self.grid.Objects[0, idx] := aEmployee;
    CritSec.Leave;

    for I := 0 to self.grid.ColCount -1 do
    begin
      CritSec.Enter;
      self.grid.Cells[i, idx] := self.cds.Fields[i].AsString;
      CritSec.Leave;
    end;
    self.cds.Next;
    inc(idx);
    sleep(27);
  end;
end;

procedure TEmployeeThread.Setcds(const Value: TClientDataset);
begin
  Fcds := Value;
end;

procedure TEmployeeThread.SetcdsFileName(const Value: string);
begin
  FcdsFileName := Value;
end;

procedure TEmployeeThread.SetEmployeeList(const Value: TObjectList<TEmployee>);
begin
  FEmployeeList := Value;
end;

procedure TEmployeeThread.Setgrid(const Value: TAdvStringGrid);
begin
  Fgrid := Value;
end;

end.
