unit uOrderClass;

interface

uses
  Classes, generics.collections, SysUtils, db, dbClient, MSaccess, advGrid;



type

  TOrderLine = class(TObject)
    private
    FDiscount: integer;
    FQuantity: integer;
    FOrderNo: integer;
    FItemNo: integer;
    FPartNo: integer;
    function getDiscountAsString: string;
    procedure SetDiscount(const Value: integer);
    procedure SetItemNo(const Value: integer);
    procedure SetOrderNo(const Value: integer);
    procedure SetPartNo(const Value: integer);
    procedure SetQuantity(const Value: integer);
    public
     property OrderNo : integer read FOrderNo write SetOrderNo;
     property ItemNo  : integer read FItemNo write SetItemNo;
     property PartNo  : integer read FPartNo write SetPartNo;
     property Quantity : integer read FQuantity write SetQuantity;
     property Discount : integer read FDiscount write SetDiscount;
     property DiscountAsString : string read getDiscountAsString;
     procedure LoadFromDataset(aDs : TDataset);

  end;


type
  TOrder = class (TObject)
    private
    FCustNo: integer;
    FShipToContact: string;
    FOrderNo: integer;
    FShipDate: TDateTime;
    FSaleDate: TDateTime;
    FShipvia: string;
    FisDirty: boolean;
    FOrderLines: TObjectList<TOrderLine>;
    FMikkel: string;

    procedure SetCustNo(const Value: integer);
    procedure SetOrderNo(const Value: integer);
    procedure SetSaleDate(const Value: TDateTime);
    procedure SetShipDate(const Value: TDateTime);
    procedure SetShipToContact(const Value: string);
    procedure SetShipvia(const Value: string);

    function getSaleDateAsString: string;
    function getShipDateAsString: string;
    procedure SetisDirty(const Value: boolean);
    procedure SetOrderLines(const Value: TObjectList<TOrderLine>);
    procedure SetMikkel(const Value: string);
    public
     property OrderNo : integer read FOrderNo write SetOrderNo;
     property CustNo  : integer read FCustNo write SetCustNo;
     property SaleDate : TDateTime read FSaleDate write SetSaleDate;
     property SaleDateAsString : string read getSaleDateAsString;
     property ShipDate : TDateTime read FShipDate write SetShipDate;
     property ShipDateAsString : string read getShipDateAsString;
     property ShipToContact : string read FShipToContact write SetShipToContact;
     property Shipvia : string read FShipvia write SetShipvia;
     property isDirty : boolean read FisDirty write SetisDirty;
     property OrderLines : TObjectList<TOrderLine> read FOrderLines write SetOrderLines;
     property Mikkel : string read FMikkel write SetMikkel;
     Function saveInDB (aConnection : TMSConnection) : integer;
     procedure LoadFromDs(aDs : TDataset);
     procedure LoadLinesFromDS (aDs : TDataset);
     constructor Create;
     destructor destroy;
  end;

type
  TOrderThread = class(TThread)
  private
    FOrderList: TObjectList<TOrder>;
    FcdsOrderLines: TClientDataset;
    FcdsOrders: TClientDataset;
    FgridOrderLines: TAdvStringGrid;
    FgridOrders: TAdvStringGrid;
    procedure SetcdsOrderLines(const Value: TClientDataset);
    procedure SetcdsOrders(const Value: TClientDataset);
    procedure SetOrderList(const Value: TObjectList<TOrder>);
    procedure SetgridOrderLines(const Value: TAdvStringGrid);
    procedure SetgridOrders(const Value: TAdvStringGrid);
    protected
      procedure Execute; override;
    public
      property cdsOrders : TClientDataset read FcdsOrders write SetcdsOrders;
      property cdsOrderLines : TClientDataset read FcdsOrderLines write SetcdsOrderLines;
      property OrderList : TObjectList<TOrder> read FOrderList write SetOrderList;
      property gridOrders : TAdvStringGrid read FgridOrders write SetgridOrders;
      property gridOrderLines : TAdvStringGrid read FgridOrderLines write SetgridOrderLines;
  end;


implementation

uses
  Orders;

{ TOrder }

constructor TOrder.Create;
begin
  self.FOrderLines := TObjectList<TOrderLine>.Create(true);
end;

destructor TOrder.destroy;
begin
  self.FOrderLines.Free;
end;

function TOrder.getSaleDateAsString: string;
begin
  if self.FSaleDate < 1 then
    result := ''
  else
    result := dateToStr(self.FSaleDate);
end;

function TOrder.getShipDateAsString: string;
begin
  if self.FShipDate < 1 then
    result := ''
  else
    result := dateToStr(self.FShipDate);
end;

procedure TOrder.LoadFromDs(aDs: TDataset);
begin
  self.FCustNo := aDs.FieldByName('CustNo').AsInteger;
  self.FShipToContact := aDs.FieldByName('ShipToContact').AsString;
  self.FOrderNo := aDs.FieldByName('OrderNo').AsInteger;
  self.FShipDate := ads.FieldByName('Shipdate').AsDateTime;
  self.FSaleDate := ads.FieldByName('SaleDate').AsDateTime;
  self.FShipvia := ads.FieldByName('ShipVia').AsString;
end;

procedure TOrder.LoadLinesFromDS(aDs: TDataset);
var
  aOrderLine : TOrderLine;
begin
  aDs.First;
  while not ads.Eof do
  begin
    aOrderLine := TOrderLine.Create;
    aOrderLine.LoadFromDataset(aDs);
    self.FOrderLines.Add(aOrderLine);
    ads.Next;
  end;
end;

function TOrder.saveInDB(aConnection: TMSConnection): integer;
var
  aQry : TMSQuery;
begin
  aQry := TMSQuery.Create(nil);
  try
    aQry.Connection := aConnection;
  finally
    aQry.Free;
  end;
end;

procedure TOrder.SetCustNo(const Value: integer);
begin
  FCustNo := Value;
end;

procedure TOrder.SetisDirty(const Value: boolean);
begin
  FisDirty := Value;
end;

procedure TOrder.SetMikkel(const Value: string);
begin
  FMikkel := Value;
end;

procedure TOrder.SetOrderLines(const Value: TObjectList<TOrderLine>);
begin
  FOrderLines := Value;
end;

procedure TOrder.SetOrderNo(const Value: integer);
begin
  FOrderNo := Value;
end;

procedure TOrder.SetSaleDate(const Value: TDateTime);
begin
  FSaleDate := Value;
end;


procedure TOrder.SetShipDate(const Value: TDateTime);
begin
  FShipDate := Value;
end;



procedure TOrder.SetShipToContact(const Value: string);
begin
  FShipToContact := Value;
end;

procedure TOrder.SetShipvia(const Value: string);
begin
  FShipvia := Value;
end;

{ TOrderLine }

function TOrderLine.getDiscountAsString: string;
begin
   if self.fdiscount = 0 then
     result := 'Ingen rabat'
   else
     result := IntToStr(self.fdiscount);
end;

procedure TOrderLine.LoadFromDataset(aDs : TDataset);
begin
  self.FDiscount := aDs.FieldByName('Discount').asInteger;
  self.FQuantity := aDs.FieldByName('Qty').AsInteger;
  self.FOrderNo  := aDs.FieldByName('OrderNo').AsInteger;
  self.FItemNo   := aDS.FieldByName('itemno').AsInteger;
  self.FPartNo   := aDs.FieldByName('PartNo').AsInteger;

end;

procedure TOrderLine.SetDiscount(const Value: integer);
begin
  FDiscount := Value;
end;

procedure TOrderLine.SetItemNo(const Value: integer);
begin
  FItemNo := Value;
end;

procedure TOrderLine.SetOrderNo(const Value: integer);
begin
  FOrderNo := Value;
end;

procedure TOrderLine.SetPartNo(const Value: integer);
begin
  FPartNo := Value;
end;

procedure TOrderLine.SetQuantity(const Value: integer);
begin
  FQuantity := Value;
end;

{ TOrderThread }

procedure TOrderThread.Execute;
var
  idx : integer;
  aOrder : TOrder;
begin
  inherited;
  self.cdsOrders := TClientDataset.Create(nil);
  self.cdsOrderLines := TClientDataset.Create(nil);
  self.cdsOrders.LoadFromFile('c:\Program Files\Common Files\CodeGear Shared\Data\Orders.xml');
  self.cdsOrderLines.LoadFromFile('c:\Program Files\Common Files\CodeGear Shared\Data\Items.xml');
  self.cdsOrders.Open;
  self.cdsOrderLines.Open;
  orderCrit.Enter;
  self.FgridOrders.RowCount := self.FcdsOrders.RecordCount + 1;
  orderCrit.Leave;
  idx := 1;
  while not self.FcdsOrders.Eof do
  begin
    aOrder := TOrder.Create;
    aOrder.LoadFromDs(self.FcdsOrders);
    self.FcdsOrderLines.Filtered := False;
    self.FcdsOrderLines.Filter := format('OrderNo = %d',[self.FcdsOrders.FieldByName('ORderNo').AsInteger]);
    self.FcdsOrderLines.Filtered := true;
    aOrder.LoadLinesFromDS(self.FcdsOrderLines);
    cdsOrders.Next;
    orderCrit.Enter;
    self.FgridOrders.Objects[0, idx] := aOrder;
    self.FgridOrders.RepaintRow(idx);
    orderCrit.Leave;
    self.FOrderList.Add(aOrder);
    idx := idx + 1;
  end;
  cdsOrderLines.Free;
  cdsOrders.Free;
end;

procedure TOrderThread.SetcdsOrderLines(const Value: TClientDataset);
begin
  FcdsOrderLines := Value;
end;

procedure TOrderThread.SetcdsOrders(const Value: TClientDataset);
begin
  FcdsOrders := Value;
end;

procedure TOrderThread.SetgridOrderLines(const Value: TAdvStringGrid);
begin
  FgridOrderLines := Value;
end;

procedure TOrderThread.SetgridOrders(const Value: TAdvStringGrid);
begin
  FgridOrders := Value;
end;

procedure TOrderThread.SetOrderList(const Value: TObjectList<TOrder>);
begin
  FOrderList := Value;
end;

end.
